"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ScannerFuncDictionary = exports.ScanCollectionForMetadataProblems = void 0;
const SchedulerOperations = __importStar(require("./SchedulerOperations"));
const Logging = __importStar(require("./Logging"));
const TeamOperations = __importStar(require("./TeamOperations"));
const Logger = Logging.GetLogger(Logging.LoggingArea.ORG_SCANNER);
/**
 * Function to be called using scheduler with --scanner arg. Arg will determine the scannerFunc to be used. Will scan all orgs within
 * given firestore org collection for a problem specified by scannerFunc
 * @param scannerFunc callback that is run for every org within the firestore to check for a specific metadata problem
 * @returns an array of objects containing orgs that have the problem specified by scannerFunc, and can include additional info
 * about the nature of the problem
 */
const ScanCollectionForMetadataProblems = (collection, scannerFunc, opts) => {
    const admin = SchedulerOperations.FirebaseExport;
    const firestore = admin.firestore();
    const orgsWithProblems = {};
    return firestore.collection(collection).listDocuments()
        .then(querySnapshot => {
        const scannerPromises = querySnapshot.map(orgDocRef => {
            return scannerFunc(orgDocRef, opts, firestore)
                .then(scanProblemInfo => {
                if (scanProblemInfo) {
                    orgsWithProblems[orgDocRef.id] = scanProblemInfo;
                }
            });
        });
        return Promise.all(scannerPromises);
    }).then(() => {
        Logger.info("finished all scanners");
        return orgsWithProblems;
    });
};
exports.ScanCollectionForMetadataProblems = ScanCollectionForMetadataProblems;
// multiple dev envs will soon be supported, this is a testing/example function and may still be useful for the near future
const dupeDevEnvScanner = (orgDocRef, cmdOptions, firestore) => {
    const workspaceCollRef = orgDocRef.collection("workspaces");
    const devWorkspaces = [];
    return workspaceCollRef.get()
        .then(workspaceSnapshot => {
        const allWorkspaceData = workspaceSnapshot.docs.map(workspaceDoc => workspaceDoc.data());
        allWorkspaceData.forEach(workspaceData => {
            if (workspaceData.devEnv) {
                devWorkspaces.push((workspaceData.id).toString());
            }
        });
        return devWorkspaces.length <= 1 ? null : { devEnvIDs: devWorkspaces };
    });
};
/*
 * run with scan -s isSuperUserRoleScanner -c users
 * scans the users docs for super users and swaps them to be admins with a hasSuperUserAccess flag equalling true
 */
const isSuperUserRoleScanner = (userDocRef, cmdOptions, firestore) => {
    return userDocRef.get()
        .then((docSnapshot) => {
        if (docSnapshot.data().role === "SuperUser") {
            Logger.info("(isSuperUserRoleScanner) super user data!", docSnapshot.data());
            userDocRef.update({
                role: "Admin",
                hasSuperUserAccess: true
            });
        }
    });
};
const stepRunResultScanner = (orgDocRef, cmdOptions, firestore) => {
    if (cmdOptions.orgID != orgDocRef.id) {
        //orgid doesnt match
        return Promise.resolve();
    }
    else {
        Logger.info("found orgID", cmdOptions.orgID, cmdOptions.nodeID);
        return new Promise((resolve, reject) => {
            let runChecks = [];
            let runResultsData = {}; //key is runID, value is runResult for the nodeiD we are looking for 
            orgDocRef.collection("runs").get()
                .then((runs) => {
                runs.docs.forEach((run) => {
                    const runResults = run.ref.collection("runResults");
                    const nodeIDRunResult = runResults.doc(cmdOptions.nodeID);
                    runChecks.push(nodeIDRunResult.get().then((document) => {
                        if (document.exists) {
                            //Logger.info("EXISTS for runID", run.id)
                            runResultsData[run.id] = document.data();
                        }
                        else {
                            //Logger.info("doesnt EXISTS for runID", run.id)
                        }
                    }));
                });
                Promise.all(runChecks).then(() => {
                    Logger.info("completed", runResultsData);
                    Object.keys(runResultsData).sort((a, b) => {
                        return parseInt(b) - parseInt(a);
                    }).forEach((runID) => {
                        console.log("runID", runID);
                        console.log("result", runResultsData[runID]);
                    });
                    resolve();
                }).catch(reject);
            });
        });
    }
};
const deleteOrgsScanner = (orgDocRef, cmdOptions, firestore) => {
    const pattern = cmdOptions.pattern;
    const dry = cmdOptions.dry;
    const orgNameRegex = new RegExp(pattern);
    if (!pattern) {
        throw new Error("no specified pattern");
    }
    return new Promise((resolve, reject) => {
        return orgDocRef.get().then((orgDoc) => {
            if (!orgDoc.data()) {
                //possible eto have incomplete data? 
                resolve();
                return;
            }
            const name = orgDoc.data().orgName;
            if (!orgNameRegex.test(name)) {
                resolve();
            }
            else {
                if (dry) {
                    Logger.info("would delete org", orgDocRef.id, " ", name);
                    resolve();
                }
                else {
                    Logger.info("deleting org", orgDocRef.id, " ", name);
                    TeamOperations.DeleteOrgAdmin(firestore, orgDocRef.id).then(resolve);
                }
            }
        }).catch((err) => {
            Logger.error("unable to get org data ofr orgID", orgDocRef.id, err);
        });
    });
};
exports.ScannerFuncDictionary = {
    dupeDevEnv: dupeDevEnvScanner,
    stepRunResult: stepRunResultScanner,
    isSuperUserRoleScanner: isSuperUserRoleScanner,
    deleteOrgs: deleteOrgsScanner
};
//# sourceMappingURL=OrgScanner.js.map